class Order:
    def __init__(self, product_name, quantity, total):
        self.name = name
        self.quantity = quantity
        self.total = total

class Customer:
    def __init__(self, name: str):
        self.name = name
        self.orders = []

    def place_order(self, product_name, quantity, order_total):
        self.orders.append(Order(product_name, quantity, order_total))
        #print(f"You spent ${order_total}")

class Product:
    def __init__(self, name, description, price, quantity):
        self.name = name
        self.description = description
        self.price = price
        self.quantity = quantity

    def get_price(self):
        return self.price

    def decrement_quantity(self, amount):
        self.quantity -= amount


class ProductRepository:
    def __init__(self):
        self.products = []

    def add_product(self, product):
        self.products.append(product)

    def remove_product(self, product):
        self.products.remove(product)

    def get_product_by_name(self, name):
        for product in self.products:
            if product.name == name:
                return product
        return None


class CustomerRepository:
    def __init__(self):
        self.customers = []

    def add_customer(self, customer):
        self.customers.append(customer)

    def remove_customer(self, customer):
        self.customers.remove(customer)

    def get_customer_by_name(self, name):
        for customer in self.customers:
            if customer.name == name:
                return customer
        return None


class OrderService:
    def __init__(self, product_repository):
        self.product_repository = product_repository

    def place_order(self, customer, product_name, quantity):
        product= self.product_repository.get_product_by_name(product_name)
        if product is None:
            raise ValueError(f"No product with name '{product_name}' found")
        if product.quantity < quantity:
            raise ValueError(f"Only {product.quantity} units of ’{product_name}' available")
        order_total = product.get_price() * quantity
        customer.place_order(product, quantity, order_total)
        product.decrement_quantity(quantity)


product_repository = ProductRepository()
product_repository.add_product(Product("Apple", "A juicy red fruit", 1.99, 10))
product_repository.add_product(Product("Banana", "A yellow fruit", 0.99, 5))

customer_repository = CustomerRepository()
customer_repository.add_customer(Customer("Pera"))
customer_repository.add_customer(Customer("Mika"))

order_service = OrderService(product_repository)

print("Welcome to the e-commerce application")

while True:
    name = input("What is your name? ")
    customer = customer_repository.get_customer_by_name(name)
    if customer : break
    
while True:
    print("Available products:")
    for product in product_repository.products:
        print(f"- {product.name}: {product.description} ({product.quantity} available) for ${product.price}")
    product_name = input("What product would you like to buy? ")
    quantity = int(input("How many units would you like to buy? "))
    try:
        order_service.place_order(customer, product_name, quantity)
        print(f"Order placed for {quantity} units of {product_name}")
    except ValueError as e:
        print(f"Error: {e}")
