<?php

function task_c(SplQueue $q) {
    while (true) {
        // Wait for a future object to be added to the queue
        while ($q->isEmpty()) {
            yield;
        }
        $future = $q->dequeue();
        if (!$future) break;
        // Wait for a random number of seconds between 1 and 5
        sleep(random_int(1, 5));
        // Generate a random value and set it on the future
        $value = random_int(1, 100);
        $future->setResult($value);
    }
}

function task_a(SplQueue $q) {
    // Create a future object
    $future = new Future();
    // Add the future object to the queue
    $q->enqueue($future);
    echo "Request sent from task_a to task_c\n";
    // Wait for the future to be set with a value by task_c
    $value = yield from $future->getResult();
    // Do something with the value
    echo "Received value from task_c in task_a: $value\n";
}

function task_b(SplQueue $q) {
    // Create a future object
    $future = new Future();
    // Add the future object to the queue
    $q->enqueue($future);
    echo "Request sent from task_b to task_c\n";
    // Wait for the future to be set with a value by task_c
    $value = yield from $future->getResult();
    // Do something with the value
    echo "Received value from task_c in task_b: $value\n";
    // signal task_c to terminate
    $q->enqueue(Null);
}

class Future {
    private $result = null;

    public function setResult($result) {
        $this->result = $result;
    }

    public function getResult() {
        while (!$this->result) {
            yield;
        } 
        return $this->result;
    }
}

// Implement cooperative multitasking
function scheduler($tasks)
{
    while (true) {
        $task = array_shift($tasks);
        if (!$task->valid() )
            if (!$tasks) break;
            else continue;
        $task->next();
        array_push($tasks, $task);
    }
}

function main() {
    // Create the queue
    $q = new SplQueue();
    // Create tasks for task_a, task_b, and task_c
    $tasks = [
        task_a($q),
        task_b($q),
        task_c($q)
    ];
    // Run the tasks

    // Start the scheduler
    scheduler($tasks);
}

// Run the main function
main();
